// ---------------------------------------------------------------------------
// Created by F. Malpartida on 9/9/12.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
// 
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file processGcode.h
// Main Gcode parser entry point.
// 
// @brief CNC gcode interpreter for 3 axis CNC machine. Configured for ELFSHL1
//     3 axis stepper motor driver shield.
//
// Acknowledgements:
// Inspired on the work of Carlos guilarte y diego colonnello (Txapuzas CNC).
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#ifndef _PROCESS_GCODE_H_
#define _PROCESS_GCODE_H_
#include "configMe.h"
#include "stepperCntrl.h"

/*!
 @function   initGcodeProcessor
 @abstract   initialize the gcode interpreter.
 @discussion Initializes the gcode interpreter and associates the stepper controller
 that will execute the commands received from the serial interface.
 @param      iController: stepper motor controller to driver the movements
 */     
void initGcodeProcessor ( stepperCntrl *iController );

/*!
 @function   processGcode
 @abstract   Performs all the gcode command processing as received from the 
 serial input.
 @discussion This is the main entry point to the gcode command parser. It
 must be called periodically since it is in charge of reading the
 gcode input from the serial line. On command reception, it enables
 stepper controller, and passes the input buffer to the main parser
 routine.
 @see        processString
 */     
void processGcode ( void );

/*!
 @function   processString
 @abstract   Parses commands (gcodes, mcodes) and executes them.
 @discussion Main gcode parser. The routine parses the received gcode and
 executes the command.
 @param      instruction: input buffer with the command to parse and execute
 @param      size: size of the string in instruction.
 */ 
static void processString(char instruction[], int size);

/*!
 @function   has_command
 @abstract   Returns true if a particular key is in the input buffer
 @discussion Searches the intruction buffer up to string_size searching
 for the keyword key. Returns true if key is in the buffer.
 @param      key: token to look for in the instruction buffer.
 @param      instruction: input string buffer.
 @param      string_size: size of the instruction buffer.
 @result     Returns true if key (character) is in instruction. false, in any
 other case.
 */
static bool has_command(char key, char instruction[], int string_size);

/*!
 @function
 @abstract   Returns the value associated to a particular key.
 @discussion The routine returns the value associated to key in the instruction.
 For example, given the gcode "X2.354", this routine will return
 2.354.
 @param      key: token to look for the in instruction buffer and to retrieve
 its value.
 @param      instruction: input string buffer
 @param      string_size: size of the input buffer.
 @result     Returns the value associate to the token keyword "key".
 */
static double search_string(char key, char instruction[], int string_size);

#endif