// ---------------------------------------------------------------------------
// Created by F. Malpartida on 9/9/12.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
// 
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file tempCntrl.h
// Main temperature controller and fan interface. Senses temperature from LM75B
// and controls the fan speed to cool the electronics.
// Requires thermistor library.
// 
// @brief 
// Main temperature controller. Uses a theremistor to monitor the on-board
// temperature and performs the fan's control loop.
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#ifndef _TEMP_CNTRL_H_
#define _TEMP_CNTRL_H_
#include "configMe.h"

#ifdef _TEMP_CONTROL_ENABLED_
#include <stdint.h>
#include <thermistor.h>
#include <lm75.h>

class tempCntrl 
{
public:
   
   /*!
    @function   
    @abstract   temperature controller constructor.
    @discussion Creates and manages the main temperature control loop.
    @param      fanPin: control pin associated to the fan.
    @param      sensorAddr: I2C base address of an LM75B
    */     
   tempCntrl ( uint8_t fanPin, uint8_t sensorAddr );
   
   /*!
    @function   init
    @abstract   initialize the electronics.
    @discussion Initialize the electronics, fan control pin and I2C LM75B
    temperature sensor. This call must be called as part of the
    initialization process and before calling any other method from
    this class.
    */
   void init ( void );
   
   /*!
    @function   controlTemp
    @abstract   main temperature control loop.
    @discussion Actively monitors the temperature and actuates over the fan speed.
    Performs temperature readings from the temperature sensor,
    and proportionally control the fan speed. Temperature threshold
    constans in "configMe.h", MAX_TEMP: maximum temperature for full
    fan speed, START_TEMP: temperature at which the fan starts.
    */
   void controlTemp ( void );
   
   /*!
    @function   tempGet
    @abstract   Returns the current on board temperature reading.
    @discussion Returns the current on board temperature reading after low pass
    filter.
    @result     Filtered temperature reading as per low pass filter in the control
    loop.
    */
   double tempGet ( void );
   
   /*!
    @function   speedGet
    @abstract   Returns the current fan speed setting.
    @discussion Gets the current fan speed setting.
    @result     Current fan speed setting.
    */   
   uint8_t speedGet ( void );
   
private:
   
   /*!
    @function   fanControl
    @abstract   Sets the fan speed based on temperature readings.
    @discussion Sets the fan speed based on temperature readings, activating
    PWM proportionally to the temperature window configured by
    START_TEMP - MAX_TEMP in configMe.h
    */
   void fanControl ( double avrTemp );
   
   uint8_t    _fanPin;      // fan control pin
   lm75       _sensor;      // temperature sensor (LM75B)
   double     _filtTemp;    // current temperature reading
   uint8_t    _speed;       // fan speed setting
};

#endif // _TEMP_CONTROL_ENABLED_

#endif